#ifndef __CODE_GENERATION_H
#define __CODE_GENERATION_H

#define INT_SIZE 2
#define POINTER_SIZE 4
//in bytes
#define UNREF_NEG "WORD PTR[BP-%d]"
#define UNREF_POS "WORD PTR[BP+%d]"
//access to integer local variables and arguments
#define RETURN_REGISTER "r_return"

//void emit(char* s);

/************************************************/
/* 1/ Arithmetic operations                     */
/************************************************/

/* result_reg = op1_reg + op2_reg       */ void add(char* result_reg, char* op1_reg, char* op2_reg);
/* result_reg = op1_reg - op2_reg       */ void sub(char* result_reg, char* op1_reg, char* op2_reg);
/* result_reg = op1_reg * op2_reg       */ void mul(char* result_reg, char* op1_reg, char* op2_reg);

/************************************************/
/* 2/ Register setting                          */
/************************************************/

/* target_reg = value                   */ void move_int(char* target_reg, int value);
/* target_reg = source_reg              */ void move_reg(char* target_reg, char* source_reg);
/* target_reg = mem                     */ void load(char* target_reg, char* mem);
/* mem = source_reg                     */ void store(char* mem, char* source_reg);
/* push reg                             */ void push(char* reg);

/************************************************/
/* 3/ Tests                                     */
/************************************************/

/* result_reg = 1 if op1_reg > op2_reg  */ void cmp_g(char* result_reg, char* op1_reg, char* op2_reg);
/*              0 otherwise             */
/* result_reg = 1 if op1_reg == op2_reg */ void cmp_eq(char* result_reg, char* op1_reg, char* op2_reg);
/*              0 otherwise             */
/* result_reg = 1 if op1_reg != op2_reg */ void cmp_neq(char* result_reg, char* op1_reg, char* op2_reg);
/*              0 otherwise             */

/************************************************/
/* 4/ Jumps                                     */
/************************************************/

/* output the label                     */ void lbl(char* label);
/* jump to label                        */ void jmp_lbl(char* label);
/* jump to label if reg = 0             */ void jmp_if_false(char* reg, char* label);

/************************************************/
/* 5/ Functions stuff                           */
/************************************************/

/* call 'function'                      */ void call(char* function);
/* allocate a stack-frame               */ void emit_prelude(int local_size);
/* deallocate the current stack-frame   */ void emit_postlude(int local_size);

#endif
