#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "symbol_table.h"
#include "register.h"
#include "code_generation.h"

void reset_symbols()
{
  nb_arguments = 0;
  nb_local_vars = 0;
}

void add_argument(char* id)
{
  argument[nb_arguments] = strdup(id);
  argument_register[nb_arguments] = new_register(); 
  //map 'id' to a new register
  nb_arguments++;  
}

void add_local_var(char* id)
{
  local_var[nb_local_vars] = strdup(id);
  local_var_register[nb_local_vars] = new_register(); 
  //map 'id' to a new register
  nb_local_vars++;  
}

char* get_register(char* id)
{
  int rank;
  if((rank=argument_rank(id)) != -1)
    //argument?
    return strdup(argument_register[rank]);

  if((rank=local_var_rank(id)) != -1)
    //local variable?
    return strdup(local_var_register[rank]);

  printf("symbol_table:emit_register: unknown symbol %s\n",id);
  exit(1);
}

char* get_address(char* id)
{
  char* result = (char*)malloc(50*sizeof(int));
  int rank;

  if((rank=argument_rank(id)) != -1)
    //argument?
    {
      int offset = 2*POINTER_SIZE + INT_SIZE*(nb_arguments - rank);
      int string_size = sprintf(result,UNREF_NEG,offset);

      if(string_size>50)
	{
	  printf("symbol_table:get_address: address out of range\n");
	  exit(1);
	}
      return result;
    }

  if((rank=local_var_rank(id)) != -1)
    //local variable?
    {
      int offset = INT_SIZE*rank;
      int string_size = sprintf(result,UNREF_POS,offset);

      if(string_size>50)
	{
	  printf("symbol_table:get_address: address out of range\n");
	  exit(1);
	}
      return result;
    }

  printf("symbol_table:get_address: unknown symbol %s\n",id);
  exit(1);
}

void emit_load()
{
  int i;

  //Load arguments
  for(i=0; i<nb_arguments; i++)
    load(argument_register[i],get_address(argument[i]));

  //Load local variables
  for(i=0; i<nb_local_vars; i++)
    load(local_var_register[i],get_address(local_var[i]));
}

void emit_store()
{
  int i;

  //Load arguments
  for(i=0; i<nb_arguments; i++)
    store(get_address(argument[i]),argument_register[i]);

  //Load local variables
  for(i=0; i<nb_local_vars; i++)
    store(get_address(local_var[i]),local_var_register[i]);
}

void print_symbols()
{
  int i;

  //Arguments
  printf("#       Arguments: ");
  for(i=0; i<nb_arguments; i++)
    printf("%s -> %s, ",argument[i],argument_register[i]);
  if(nb_arguments == 0)
    printf("none.");
  printf("\n");

  //Local variables
  printf("# Local variables: ");
  for(i=0; i<nb_local_vars; i++)
    printf("%s -> %s, ",local_var[i],local_var_register[i]);
  if(nb_local_vars == 0)
    printf("none.");
  printf("\n");
}


//Private
int argument_rank(char* id)
{
  int i;
  for(i=0; i<nb_arguments; i++)
    if(strcmp(id,argument[i]) == 0)
      return i;
  return -1;
}

//Private
int local_var_rank(char* id)
{
  int i;
  for(i=0; i<nb_local_vars; i++)
    if(strcmp(id,local_var[i]) == 0)
      return i;
  return -1;
}
